/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 *
 * You should have find a copy of NFSChina License. If not, please contact us
 * by email <os_support@nfschina.com>
 *
**/

/*
    SPDX-FileCopyrightText: 2014-2015 Harald Sitter <sitter@kde.org>
    SPDX-FileCopyrightText: 2019 Sefa Eyeoglu <contact@scrumplex.net>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

import QtQuick 2.0
import QtQuick.Controls 2.0
import QtQuick.Layouts 1.0
import QtQuick.Controls 2.3 as QtControls
import org.kde.plasma.core 2.1 as PlasmaCore
import org.kde.kirigami 2.5 as Kirigami
import org.kde.plasma.private.volume 0.1

ColumnLayout {
    id: delegate
    spacing: Kirigami.Units.smallSpacing
    width: parent.width

    property bool isPlayback: type.substring(0, 4) == "sink"

    readonly property var currentPort: Ports[ActivePortIndex]

    opacity: (Ports.length === 1 && Ports[0].availability == Port.Unavailable) ? 0.5 : 1

    property var pulseObject: model.PulseObject

    property alias portVisible: portBox.visible
    property bool comboBoxLabelsVisible: false
    property bool isNoiseRestraint: false

    PlasmaCore.DataSource {
        id: checkNoiseRestraint
        engine: "executable"
        connectedSources: []
        onNewData: {
            disconnectSource(sourceName)
            isNoiseRestraint = data["stdout"].toString().includes("echo-cancel");
            control.checked = isNoiseRestraint
            console.log("checkNoiseRestraint value:",isNoiseRestraint);
        }

        function check() {
            checkNoiseRestraint.connectSource("pactl list sources")
        }
    }

    PlasmaCore.DataSource {
        id: execCmd
        engine: "executable"
        connectedSources: []
        onNewData: {
            disconnectSource(sourceName)
            checkNoiseRestraint.check()
        }

        function exec(cmd) {
            execCmd.connectSource(cmd)
        }
    }

    RowLayout {
        spacing: Kirigami.Units.smallSpacing
        Layout.fillWidth: true
        // To keep the top row the same height as the row below, and stop things moving too much
        // as the KCM loads (and the port/profile ComboBoxes may appear)
        //Layout.minimumHeight: balanceButton.implicitHeight

        RadioButton {
            id: defaultButton
            Layout.fillWidth: true
            Layout.leftMargin: LayoutMirroring.enabled ? 0 : Math.round((muteButton.width - defaultButton.indicator.width) / 2)
            Layout.rightMargin: LayoutMirroring.enabled ? Math.round((muteButton.width - defaultButton.indicator.width) / 2) : 0
            spacing: Kirigami.Units.smallSpacing + Math.round((muteButton.width - defaultButton.indicator.width) / 2)
            checked: Default
            visible: delegate.ListView.view.count > 1
            onClicked: Default = true
            text: !currentPort ? Description : i18ndc("kcm_pulseaudio", "label of device items", "%1 (%2)", currentPort.description, Description)

            ToolTip {
                text: parent.text
                visible: parent.visible && parent.contentItem.truncated && labelMouseArea.containsMouse
            }
            MouseArea {
                id: labelMouseArea
                anchors.fill: parent.contentItem
                hoverEnabled: true
                acceptedButtons: Qt.NoButton
                z: defaultButton.z + 1
            }
        }

        Label {
            id: soloLabel
            Layout.fillWidth: true
            text: currentPort.description
            visible: delegate.ListView.view.count === 1
            elide: Text.ElideRight

            ToolTip {
                text: defaultButton.text
                visible: parent.visible && soloLabelMouseArea.containsMouse
            }
            MouseArea {
                id: soloLabelMouseArea
                anchors.fill: parent
                hoverEnabled: true
            }
        }

        Item {
            Layout.leftMargin: Kirigami.Units.largeSpacing
            Layout.fillWidth: true
        }

        Label {
            id: portLabel
            visible: comboBoxLabelsVisible && portBox.visible
            text: i18nd("kcm_pulseaudio", "Port:")
            Layout.leftMargin: Kirigami.Units.largeSpacing
        }

        ComboBox {
            id: portBox

            readonly property var ports: Ports

            Layout.preferredWidth: Kirigami.Units.gridUnit * 10
            visible: portBox.count > 1

            onModelChanged: currentIndex = ActivePortIndex
            currentIndex: ActivePortIndex
            onActivated: ActivePortIndex = index

            onPortsChanged: {
                var items = [];
                for (var i = 0; i < ports.length; ++i) {
                    var port = ports[i];
                    var text;
                    if (port.availability == Port.Unavailable) {
                        if (port.name == "analog-output-speaker" || port.name == "analog-input-microphone-internal") {
                            text = i18ndc("kcm_pulseaudio", "Port is unavailable", "%1 (unavailable)", port.description);
                        } else {
                            text = i18ndc("kcm_pulseaudio", "Port is unplugged", "%1 (unplugged)", port.description);
                        }
                    } else {
                        text = port.description;
                    }
                    items.push(text);
                }
                model = items;
            }

            // Ensure the popup is sufficiently wide to list *full* device names
            // and is correctly positioned to not go off the page
            popup.width: Math.max(implicitWidth, width)
            popup.leftMargin: delegate.ListView.view.Layout.leftMargin
            popup.rightMargin: delegate.ListView.view.Layout.rightMargin
            popup.x: mirrored ? 0 : width - popup.width
        }

        Label {
            id: profileLabel
            visible: comboBoxLabelsVisible && profileBox.visible
            text: i18ndc("kcm_pulseaudio", "@label", "Profile:")
            Layout.leftMargin: Kirigami.Units.largeSpacing
        }

        ComboBox {
            id: profileBox

            readonly property var card: paCardModel.data(paCardModel.indexOfCardNumber(CardIndex), paCardModel.role("PulseObject"))

            Layout.preferredWidth: Kirigami.Units.gridUnit * 12
            visible: profileBox.count > 1
            textRole: "description"

            model: card ? card.profiles.filter(profile => profile.availability === Profile.Available) : []
            currentIndex: card ? model.indexOf(card.profiles[card.activeProfileIndex]) : -1

            onActivated: index => card.activeProfileIndex = card.profiles.indexOf(model[index])

            // Ensure the popup is sufficiently wide to list *full* device names
            // and is correctly positioned to not go off the page
            popup.width: Math.max(implicitWidth, width)
            popup.leftMargin: delegate.ListView.view.Layout.leftMargin
            popup.rightMargin: delegate.ListView.view.Layout.rightMargin
            popup.x: mirrored ? 0 : width - popup.width
        }
    }

    RowLayout {
        spacing: Kirigami.Units.smallSpacing

        Label {
            id: volumeLabel
            text: i18nd("kcm_pulseaudio", "Volume")
        }

        Item {
            Layout.fillWidth: true
        }

        MuteButton {
            id: muteButton
            Layout.alignment: Qt.AlignTop | Qt.AlignVCenter
            muted: Muted
            onCheckedChanged: Muted = checked
            toolTipText: !currentPort ? Description : currentPort.description
        }

        VolumeSlider {
            id: volumeSlider
            Layout.columnSpan: 2
            Layout.alignment: Qt.AlignVCenter

            value: Volume
            onMoved: {
                Volume = value;
                Muted = (value === 0);
            }
        }

//        GridLayout {
//            rows: Math.max(1, channelSliderRepeater.count)
//            flow: GridLayout.TopToBottom

//            VolumeSlider {
//                id: volumeSlider
//                Layout.columnSpan: 2
//                Layout.alignment: Qt.AlignVCenter
//                visible: !balanceButton.checked

//                value: Volume
//                onMoved: {
//                    Volume = value;
//                    Muted = (value === 0);
//                }
//            }

//            Repeater {
//                model: balanceButton.checked ? Channels : null

//                Label {
//                    Layout.alignment: Qt.AlignTop | Qt.AlignRight
//                    text: i18ndc("kcm_pulseaudio", "Placeholder is channel name", "%1:", modelData)
//                }
//            }

//            Repeater {
//                id: channelSliderRepeater
//                model: balanceButton.checked ? RawChannels : null

//                VolumeSlider {
//                    Layout.fillWidth: true

//                    value: ChannelVolumes[index]
//                    onMoved: {
//                        delegate.pulseObject.setChannelVolume(index, value);

//                        // volumes are updated async, so we'll just assume it worked here
//                        let newChannelVolumes = ChannelVolumes;
//                        newChannelVolumes[index] = value;
//                        Muted = newChannelVolumes.every((volume) => {
//                            return volume === 0;
//                        });
//                    }
//                }
//            }
//        }

        Item {
            Layout.leftMargin: Kirigami.Units.smallSpacing
        }

        Button {
            Layout.alignment: Qt.AlignTop
            Layout.topMargin: 6
            id: testButton
            text: i18ndc("kcm_pulseaudio", "Perform an audio test of the device", "Test")
            icon.name: "audio-speakers-symbolic"
            visible: isPlayback
            onClicked: testOverlay.testSink(index);
        }

//        Button {
//            Layout.alignment: Qt.AlignTop
//            id: balanceButton
//            text: i18ndc("kcm_pulseaudio", "Audio balance (e.g. control left/right volume individually", "Balance")
//            icon.name: "view-media-equalizer"
//            checkable: true

//            onClicked: {
//                if (checked) {
//                    return;
//                }

//                // When unifying channels again, set all of them to the same value
//                // as to not keep an odd difference when adjusting them in unison
//                let volumes = ChannelVolumes;
//                for (let i = 0, count = RawChannels.length; i < count; ++i) {
//                    volumes[i] = Volume;
//                }
//                // NOTE "ChannelVolumes = volumes" does not work as the
//                // AbstractModel does not have the magic JS Array to Qt List
//                // conversion stuff and only sees our Array as a QJSValue
//                delegate.pulseObject.channelVolumes = volumes;
//            }

//            // Default to individual mode when a channel has a different volume
//            Component.onCompleted: {
//                checked = ChannelVolumes.some((volume) => {
//                    return volume !== ChannelVolumes[0];
//                });
//            }
//        }
    }

//    RowLayout {
//        spacing: Kirigami.Units.smallSpacing
//        height: 47

//        Label {
//            id: volumeBalanceLabel
//            text: i18nd("kcm_pulseaudio", "Volume Feedback")
//        }

//        KcmSlider {
//            id: balanceSlider
//            from: 0
//            to: 100
//            value: 50
//            Layout.fillWidth: true
//            showCompletColor: false

//            //            function load() {
//            //                enabled = device.supportsPointerAcceleration
//            //                if (!enabled) {
//            //                    value = 0.2
//            //                    return
//            //                }
//            //                // transform libinput's pointer acceleration range [-1, 1] to slider range [1, 11]
//            //                //value = 4.5 * device.pointerAcceleration + 5.5
//            //                value = 6 + device.pointerAcceleration / 0.2
//            //            }

//            onValueChanged: {
//                let left1 = value > 50 ? (100 - value) / 50 * 100 : 100
//                let right1 = value < 50 ? value / 50 * 100 : 100

//                console.log("value:", value)
//                console.log("left:", left1)
//                console.log("right:", right1)
//            }
//        }
//    }

    RowLayout {
        spacing: Kirigami.Units.smallSpacing
        height: 47

        Label {
            id: volumeIncreaseLabel
            text: i18nd("kcm_pulseaudio", "Noise Restraint")
        }

        Item {
            Layout.fillWidth: true
        }

        QtControls.Switch {
            id: control
            anchors.verticalCenter:parent.verticalCenter
            anchors.right: parent.right
            checked: false
            onClicked: {
                if(checked){
                    execCmd.exec("pactl load-module module-echo-cancel")
                }else{
                    execCmd.exec("pactl unload-module module-echo-cancel")
                }
            }
            indicator: Rectangle {
                implicitWidth: 36
                implicitHeight: 20
                radius: 10
                color: control.checked ? "#007AFF" : "#808080"
                Rectangle {
                    id : smallRect
                    anchors.verticalCenter:parent.verticalCenter
                    width: 16
                    height: 16
                    radius: 8
                    color: control.down ? "#cccccc" : "#ffffff"
                    border.color: control.checked ? (control.down ? "#17a81a" : "#007AFF") : "#999999"
                    NumberAnimation on x{
                        to: control.width - smallRect.width-7
                        running: control.checked ? true : false
                        duration: 200
                    }
                    NumberAnimation on x{
                        to: 0
                        running: control.checked ? false : true
                        duration: 200
                    }
                }
            }

            Component.onCompleted: {
                checkNoiseRestraint.check()
            }
        }
    }

    Kirigami.Separator {
        visible: (delegate.ListView.view.count != 0) && (delegate.ListView.view.count != (index + 1))
        Layout.topMargin: Kirigami.Units.smallSpacing
        Layout.fillWidth: true
    }
}
