/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 *
 * You should have find a copy of NFSChina License. If not, please contact us
 * by email <os_support@nfschina.com>
 *
**/

/*
    SPDX-FileCopyrightText: 2014-2015 Harald Sitter <sitter@kde.org>
    SPDX-FileCopyrightText: 2016 David Rosca <nowrep@gmail.com>
    SPDX-FileCopyrightText: 2019 Sefa Eyeoglu <contact@scrumplex.net>
    SPDX-FileCopyrightText: 2020 Nicolas Fella <nicolas.fella@gmx.de>
    SPDX-FileCopyrightText: 2021 Ismael Asensio <isma.af@gmail.com>

    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
*/

import QtQuick 2.7
import QtQuick.Layouts 1.3
import QtQuick.Controls 2.0

import org.kde.kcoreaddons 1.0 as KCoreAddons
import org.kde.kcm 1.3
import org.kde.kcm 1.6 as KCM
import org.kde.kirigami 2.13 as Kirigami
import org.kde.kquickcontrols 2.0 as KQuickControls
import org.kde.plasma.core 2.1 as PlasmaCore
import org.kde.plasma.private.volume 0.1
import QtGraphicalEffects 1.15


KCM.SimpleKCM {
    id: root

    property bool lighttheme:PlasmaCore.Theme.viewBackgroundColor=="#ffffff"
    property var backgroundcolor: PlasmaCore.Theme.viewBackgroundColor
    property var textcolor:PlasmaCore.Theme.textColor
    property var bordercolor:lighttheme?"#CCCCCC":"#2D2D2D"
    property var spaceline:lighttheme?"#F0F0F0":"#2D2D2D"
    property int leftAndRightMargin: 24

    // DEFAULT_SINK_NAME in module-always-sink.c
    readonly property string dummyOutputName: "auto_null"
    property int maxVolumeValue: PulseAudio.NormalVolume // the applet supports changing this value. We will just assume 65536 (100%)
    ConfigModule.quickHelp: i18nd("kcm_pulseaudio", "This module allows configuring the Pulseaudio sound subsystem.")

    property int currentMaxVolumePercent: config.raiseMaximumVolume ? 150 : 100
    property int currentMaxVolumeValue: currentMaxVolumePercent * PulseAudio.NormalVolume / 100.00

    implicitHeight: Kirigami.Units.gridUnit * 50
    implicitWidth: Kirigami.Units.gridUnit * 28


    GlobalConfig {
        id: config
    }

    SinkModel {
        id: paSinkModel
    }

    SourceModel {
        id: paSourceModel
    }

    VolumeFeedback {
        id: feedback
    }

    CardModel {
        id: paCardModel

        function indexOfCardNumber(cardNumber) {
            for (var idx = 0; idx < count; idx++) {
                if (data(index(idx, 0), role("Index")) == cardNumber) {
                    return index(idx, 0);
                }
            }
            return index(-1, 0);
        }
    }

    PulseObjectFilterModel {
        id: paSinkFilterModel

        sortRole: "SortByDefault"
        sortOrder: Qt.DescendingOrder
        filterOutInactiveDevices: true
        sourceModel: paSinkModel
    }

    PulseObjectFilterModel {
        id: paSourceFilterModel

        sortRole: "SortByDefault"
        sortOrder: Qt.DescendingOrder
        filterOutInactiveDevices: true
        sourceModel: paSourceModel
    }

    ModuleManager {
        id: moduleManager
    }

    // Only show labels if both port/profile comboboxes are visible
    readonly property var comboBoxLabelsVisible: {
        for (var i = 0; i < sinks.count; ++i) {
            let delegate = sinks.itemAtIndex(i)
            if (delegate != null && delegate.portVisible)
                return true
        }
        for (var i = 0; i < sources.count; ++i) {
            let delegate = sources.itemAtIndex(i)
            if (delegate != null && delegate.portVisible)
                return true
        }
        return false
    }

    function enableGlobalMute() {
        var role = paSinkModel.role("Muted");
        var rowCount = paSinkModel.rowCount();
        // List for devices that are already muted. Will use to keep muted after disable GlobalMute.
        var globalMuteDevices = [];

        for (var i = 0; i < rowCount; i++) {
            var idx = paSinkModel.index(i, 0);
            var name = paSinkModel.data(idx, paSinkModel.role("Name"));
            if (paSinkModel.data(idx, role) === false) {
                paSinkModel.setData(idx, true, role);
            } else {
                globalMuteDevices.push(name + "." + paSinkModel.data(idx, paSinkModel.role("ActivePortIndex")));
            }
        }
        // If all the devices were muted, will unmute them all with disable GlobalMute.
        config.globalMuteDevices = globalMuteDevices.length < rowCount ? globalMuteDevices : [];
        config.globalMute = true;
        config.save();
    }

    function disableGlobalMute() {
        var role = paSinkModel.role("Muted");
        for (var i = 0; i < paSinkModel.rowCount(); i++) {
            var idx = paSinkModel.index(i, 0);
            var name = paSinkModel.data(idx, paSinkModel.role("Name")) + "." + paSinkModel.data(idx, paSinkModel.role("ActivePortIndex"));
            if (config.globalMuteDevices.indexOf(name) === -1) {
                paSinkModel.setData(idx, false, role);
            }
        }
        config.globalMuteDevices = [];
        config.globalMute = false;
        config.save();
    }

    function isDummyOutput(output) {
        return output && output.name === dummyOutputName;
    }

    function volumePercent(volume) {
        return Math.round(volume / PulseAudio.NormalVolume * 100.0);
    }

    function boundVolume(volume) {
        return Math.max(PulseAudio.MinimalVolume, Math.min(volume, currentMaxVolumeValue));
    }

    // Increment a VolumeObject (Sink or Source) by %volume.
    function changeVolumeByPercent(volumeObject, deltaPercent) {
        const oldVolume = volumeObject.volume;
        const oldPercent = volumePercent(oldVolume);
        const targetPercent = oldPercent + deltaPercent;
        const newVolume = boundVolume(Math.round(PulseAudio.NormalVolume * (targetPercent/100)));
        const newPercent = volumePercent(newVolume);
        volumeObject.muted = newPercent == 0;
        volumeObject.volume = newVolume;
    }

    // Increment the preferredSink by %volume.
    function changeSpeakerVolume(deltaPercent) {
        if (!paSinkModel.preferredSink || isDummyOutput(paSinkModel.preferredSink)) {
            return;
        }
        changeVolumeByPercent(paSinkModel.preferredSink, deltaPercent);
        playFeedback();
    }

    function increaseVolume() {
        if (config.globalMute) {
            disableGlobalMute();
        }
        changeSpeakerVolume(config.volumeStep);
    }

    function decreaseVolume() {
        if (config.globalMute) {
            disableGlobalMute();
        }
        changeSpeakerVolume(-config.volumeStep);
    }

    function playFeedback(sinkIndex) {
        if (!config.audioFeedback) {
            return;
        }

        if (sinkIndex == undefined) {
            sinkIndex = paSinkModel.preferredSink.index;
        }
        feedback.play(sinkIndex);
    }

    Item {
        id: item
        height: parent.height

        ColumnLayout {
            anchors {
                fill: parent
                topMargin: Kirigami.Units.largeSpacing
                leftMargin: Kirigami.Units.largeSpacing
                rightMargin: Kirigami.Units.largeSpacing
            }
            Rectangle {
                anchors {
                    fill: parent
                }

                // outputs
                Text {
                    id: outputsLabel
                    visible: sinks.visible
                    height: sinks.visible ? 20 : 0
                    anchors.left: parent.left
                    text: i18nd("kcm_pulseaudio", "Outputs")
                    // font.pixelSize: 14
                    color: textcolor
                    font.family: "SourceHanSansSC-Regular"
                }

                Rectangle {
                    id: outputs
                    anchors {
                        top: outputsLabel.bottom
                        topMargin: sinks.visible ? 15 : 0
                        left: parent.left
                        right: parent.right
                    }
                    Layout.fillWidth: true
                    color: backgroundcolor
                    height: sinks.height
                    border.color: bordercolor
                    border.width: 1
                    radius: 10

                    ListView {
                        id: sinks
                        visible: count > 0
                        anchors {
                            verticalCenter: parent.verticalCenter
                            top: parent.top
                            bottom: parent.bottom
                            left: parent.left
                            right: parent.right
                            leftMargin: leftAndRightMargin
                            rightMargin: leftAndRightMargin
                            topMargin: 16
                            bottomMargin: 16
                        }
                        interactive: false
                        spacing: Kirigami.Units.largeSpacing
                        model: paSinkModel  // show all devices
                        delegate: OutputDeviceListItem {
                            isPlayback: true
                            comboBoxLabelsVisible: comboBoxLabelsVisible

                        }

                        onCountChanged: {
                            sinks.height = count > 0 ? count * currentItem.height : 0
                            outputs.height = count > 0 ? count * currentItem.height + 2 * 16 : 0
                        }
                    }
                }

                // inputs
                Text {
                    id: inputsLabel
                    visible: sources.visible
                    height: sources.visible ? 20 : 0
                    anchors {
                        top: outputs.bottom
                        left: parent.left
                        topMargin: sources.visible ? 15 : 0
                    }
                    text: i18nd("kcm_pulseaudio", "Inputs")
                    // font.pixelSize: 14
                    color: textcolor
                    font.family: "SourceHanSansSC-Regular"
                }

                Rectangle {
                    id: inputs
                    anchors {
                        top: inputsLabel.bottom
                        topMargin: sources.visible ? 15 : 0
                        left: parent.left
                        right: parent.right
                    }
                    color: backgroundcolor
                    height: sources.height
                    border.color: bordercolor
                    border.width: 1
                    radius: 10

                    ListView {
                        id: sources
                        visible: count > 0
                        anchors {
                            verticalCenter: parent.verticalCenter
                            top: parent.top
                            bottom: parent.bottom
                            left: parent.left
                            right: parent.right
                            leftMargin: leftAndRightMargin
                            rightMargin: leftAndRightMargin
                            topMargin: 16
                            bottomMargin: 16
                        }
                        interactive: false
                        spacing: Kirigami.Units.largeSpacing
                        model:  paSourceModel   // show all devices
                        delegate: InputDeviceListItem {
                            isPlayback: false
                            comboBoxLabelsVisible: comboBoxLabelsVisible
                        }

                        onCountChanged: {
                            sources.height = count > 0 ? count * currentItem.height : 0
                            inputs.height = count > 0 ? count * currentItem.height + 2 * 16 : 0
                        }
                    }
                }

                // customize
                Text {
                    id: customize
                    anchors {
                        top: inputs.bottom
                        left: parent.left
                        topMargin: 15
                    }
                    text: i18nd("kcm_pulseaudio", "Customize")
                    // font.pixelSize: 14
                    color: textcolor
                    font.family: "SourceHanSansSC-Regular"
                }

                Rectangle {
                    id: customizeRect
                    anchors {
                        top: customize.bottom
                        topMargin: sources.visible ? 15 : 0
                        left: parent.left
                        right: parent.right
                    }
                    color: backgroundcolor
                    height: 95
                    border.color: bordercolor
                    border.width: 1
                    radius: 10

                    clip: true
                    layer.enabled: true
                    layer.effect: OpacityMask {
                        maskSource: Rectangle {
                            width: customizeRect.width
                            height: customizeRect.height
                            radius: customizeRect.radius
                        }
                    }

                    // system sounds
                    Rectangle {
                        id: systemSounds
                        color: "transparent"

                        anchors {
                            top: parent.top
                            left: parent.left
                            right: parent.right
                        }
                        height: 47

                        Text {
                            anchors.verticalCenter: systemSounds.verticalCenter
                            anchors.left: parent.left
                            anchors.leftMargin: leftAndRightMargin
                            text: i18nd("kcm_pulseaudio", "System Sounds")
                            // font.pixelSize: 14
                            color: textcolor
                            font.family: "SourceHanSansSC-Regular"
                        }
                        Kirigami.Icon {
                            anchors.verticalCenter: systemSounds.verticalCenter
                            anchors.right: parent.right
                            anchors.rightMargin: leftAndRightMargin

                            height: 16
                            width: 16
                            source: "go-next-symbolic"
                        }

                        Rectangle {
                            id: systemSoundsBackgrund
                            anchors.fill: parent
                            opacity: 0.2
                            z: 1
                            color: "transparent"
                        }
                        MouseArea {
                            anchors.fill: parent
                            hoverEnabled: true
                            visible:true
                            z: 100
                            onEntered: {
                                systemSoundsBackgrund.color = "gray"
                            }
                            onExited: {
                                systemSoundsBackgrund.color = "transparent"
                            }
                            onClicked: {
                                systemSoundsBackgrund.color = "transparent"
                                kcm.push("SystemSounds.qml", { "config": config })
                            }
                        }
                    }

                    Rectangle{
                        id: customizeSeparator
                        anchors.top: parent.top
                        anchors.topMargin: 47
                        width:parent.width
                        color:spaceline
                        height:1
                    }

                    // app sounds
                    Rectangle {
                        id: appSounds
                        color: "transparent"
                        anchors {
                            top: customizeSeparator.bottom
                            left: parent.left
                            right: parent.right
                        }
                        height: 47

                        Text {
                            anchors.verticalCenter: appSounds.verticalCenter
                            anchors.left: parent.left
                            anchors.leftMargin: leftAndRightMargin
                            text: i18nd("kcm_pulseaudio", "Apps Sound")
                            // font.pixelSize: 14
                            color: textcolor
                            font.family: "SourceHanSansSC-Regular"
                        }
                        Kirigami.Icon {
                            anchors.verticalCenter: appSounds.verticalCenter
                            anchors.right: parent.right
                            anchors.rightMargin: leftAndRightMargin

                            height: 16
                            width: 16
                            source: "go-next-symbolic"
                        }
                        Rectangle {
                            id: appSoundsBackgrund
                            anchors.fill: parent
                            opacity: 0.2
                            z: 1
                            color: "transparent"
                        }
                        MouseArea {
                            anchors.fill: parent
                            hoverEnabled: true
                            visible:true
                            z: 100
                            onEntered: {
                                appSoundsBackgrund.color = "gray"
                            }
                            onExited: {
                                appSoundsBackgrund.color = "transparent"
                            }
                            onClicked: {
                                appSoundsBackgrund.color = "transparent"
                                kcm.push("AppsSound.qml", { "config": config, "paSinkModel": paSinkModel})
                            }
                        }
                    }
                }

                // inactive cards
                Text {
                    id: inactiveCardsLabel
                    visible: inactiveCards.visible
                    height: inactiveCards.visible ? 20 : 0
                    anchors {
                        top: customizeRect.bottom
                        left: parent.left
                        topMargin: inactiveCards.visible ? 15 : 0
                    }
                    text: i18nd("kcm_pulseaudio", "Inactive Cards")
                    // font.pixelSize: 14
                    color: textcolor
                    font.family: "SourceHanSansSC-Regular"
                }

                Rectangle {
                    id: inactiveCardsRect
                    anchors {
                        top: inactiveCardsLabel.bottom
                        topMargin: inactiveCards.visible ? 15 : 0
                        left: parent.left
                        right: parent.right
                    }
                    color: backgroundcolor
                    // height: inactiveCards.height
                    border.color: bordercolor
                    border.width: 1
                    radius: 10

                    ListView {
                        id: inactiveCards
                        visible: count > 0
                        anchors {
                            verticalCenter: parent.verticalCenter
                            left: parent.left
                            right: parent.right
                            leftMargin: leftAndRightMargin
                            rightMargin: leftAndRightMargin
                        }
                        interactive: false
                        spacing: Kirigami.Units.largeSpacing
                        model: PlasmaCore.SortFilterModel {
                            sourceModel: paCardModel
                            function role(name) {
                                return sourceModel.role(name);
                            }
                            filterCallback: function(source_row, value) {
                                let idx = sourceModel.index(source_row, 0);
                                let profiles = sourceModel.data(idx, sourceModel.role("Profiles"))
                                let activeProfileIndex = sourceModel.data(idx, sourceModel.role("ActiveProfileIndex"))
                                return profiles[activeProfileIndex].name == "off";
                            }
                        }
                        delegate: CardListItem {
                            comboBoxLabelsVisible: comboBoxLabelsVisible
                        }

                        onCountChanged: {
                            inactiveCards.height = count > 0 ? count * currentItem.height : 0
                            inactiveCardsRect.height = count > 0 ? count * currentItem.height + 2 * 6 : 0
                        }
                    }
                }




                //            // recordingStream
                //            Text {
                //                id: recordingStreamLabel
                //                visible: sinkInputView.visible
                //                height: sourceOutputView.visible ? 20 : 0
                //                anchors.top: sinkInputViewRect.bottom
                //                anchors.topMargin: sinkInputView.visible ? 6 : 0
                //                anchors.left: parent.left
                //                text: i18nd("kcm_pulseaudio", "Recording Streams")
                //                font.pixelSize: 14
                //                color: textcolor
                //                font.family: "SourceHanSansSC-Regular"
                //            }

                //            Rectangle {
                //                id: sourceOutputViewRect
                //                anchors.top: recordingStreamLabel.bottom
                //                anchors.topMargin: sourceOutputView.visible ? 6 : 0
                //                anchors.horizontalCenter:parent.horizontalCenter
                //                color: backgroundcolor
                //                height: sourceOutputView.visible ? 50 : 0
                //                width: 558
                //                border.color: bordercolor
                //                border.width: 1
                //                radius: 10

                //                ListView {
                //                    id: sourceOutputView
                //                    visible: count > 0
                //                    Layout.fillWidth: true
                //                    Layout.leftMargin: Kirigami.Units.largeSpacing
                //                    Layout.rightMargin: Kirigami.Units.largeSpacing
                //                    Layout.topMargin: Kirigami.Units.smallSpacing
                //                    Layout.bottomMargin: Kirigami.Units.smallSpacing
                //                    Layout.preferredHeight: contentHeight
                //                    interactive: false
                //                    spacing: Kirigami.Units.largeSpacing
                //                    model: PulseObjectFilterModel {
                //                        filters: [ { role: "VirtualStream", value: false } ]
                //                        sourceModel: SourceOutputModel {}
                //                    }

                //                    delegate: StreamListItem {
                //                        deviceModel: sourceModel
                //                        isPlayback: false
                //                    }
                //                }
                //            }


                //                RowLayout {
                //                    Item {
                //                        Layout.fillWidth: true
                //                    }

                //                    Button {
                //                        text: i18n("Configure Volume Controls…")
                //                        icon.name: "configure"
                //                        onClicked: kcm.push("VolumeControlsConfig.qml", { "config": config })
                //                    }

                //                    Button {
                //                        id: configureButton
                //                        visible: moduleManager.settingsSupported
                //                        enabled: moduleManager.configModuleLoaded
                //                        text: i18nd("kcm_pulseaudio", "Configure…")
                //                        icon.name: "configure"

                //                        ToolTip.visible: !enabled && hovered
                //                        ToolTip.text: i18nd("kcm_pulseaudio", "Requires %1 PulseAudio module", moduleManager.configModuleName)

                //                        checkable: true
                //                        checked: configureMenu.opened
                //                        onClicked: configureMenu.open()

                //                        Menu {
                //                            id: configureMenu
                //                            y: -height - Kirigami.Units.smallSpacing
                //                            modal: true

                //                            MenuItem {
                //                                text: i18nd("kcm_pulseaudio", "Add virtual output device for simultaneous output on all local sound cards")
                //                                checkable: true
                //                                checked: moduleManager.combineSinks
                //                                onToggled: moduleManager.combineSinks = checked;
                //                            }
                //                            MenuItem {
                //                                text: i18nd("kcm_pulseaudio", "Automatically switch all running streams when a new output becomes available")
                //                                checkable: true
                //                                checked: moduleManager.switchOnConnect
                //                                onToggled: moduleManager.switchOnConnect = checked;
                //                            }
                //                        }
                //                    }
                //                }
            }
        }
    }
    SpeakerTest {
        id: tester
        sink: testOverlay.sinkObject

        onShowErrorMessage: message => {
                                testError.text = i18ndc("kcm_pulseaudio",
                                                        "%1 is an error string produced by an external component, and probably untranslated",
                                                        "Error trying to play a test sound. \nThe system said: \"%1\"", message);
                                testError.visible = true;
                            }
    }

    Kirigami.OverlaySheet {
        id: testOverlay

        property var sinkObject: null
        property string description: ""
        property string iconName: "audio-card"
        property string profile: ""
        property string port: ""

        function testSink(index) {
            let modelIndex = sinks.model.index(Math.max(index, 0), 0);
            sinkObject = sinks.model.data(modelIndex, sinks.model.role("PulseObject"));
            description = sinks.model.data(modelIndex, sinks.model.role("Description"));
            iconName = sinks.model.data(modelIndex, sinks.model.role("IconName")) || "audio-card";

            let ports = sinks.model.data(modelIndex, sinks.model.role("Ports"));
            port = ports.length > 1 ? ports[sinks.model.data(modelIndex, sinks.model.role("ActivePortIndex"))].description : "";

            profile = "";
            let cardIndex = paCardModel.indexOfCardNumber(sinks.model.data(modelIndex, sinks.model.role("CardIndex")));
            if (cardIndex.valid) {
                let profiles = paCardModel.data(cardIndex, paCardModel.role("Profiles")) || [];
                profile = profiles.length > 1 ? profiles[paCardModel.data(cardIndex, paCardModel.role("ActiveProfileIndex"))].description : "";
            }

            testOverlay.open();
        }

        header: GridLayout {
            columns: 2
            rowSpacing: Kirigami.Units.smallSpacing

            Kirigami.Icon {
                source: testOverlay.iconName || "audio-card"
                Layout.rowSpan: 3
                Layout.alignment: Qt.AlignCenter
            }
            Label {
                text: testOverlay.description
                font.bold: true
                Layout.fillWidth: true
                wrapMode: Text.WordWrap
            }
            Label {
                text: {
                    if (testOverlay.port.length === 0) { return testOverlay.profile }
                    if (testOverlay.profile.length === 0) { return testOverlay.port }
                    return testOverlay.profile + " / " + testOverlay.port
                }
                visible: text.length > 0
                Layout.fillWidth: true
                elide: Text.ElideRight
            }
        }

        ColumnLayout {
            Kirigami.InlineMessage {
                id: testError
                type: Kirigami.MessageType.Error
                showCloseButton: true
                Layout.fillWidth: true
            }

            GridLayout {
                columns: 3
                rowSpacing: Kirigami.Units.gridUnit

                LayoutMirroring.enabled: false  // To preserve spacial layout on RTL

                Kirigami.Avatar {
                    KCoreAddons.KUser {
                        id: kuser
                    }
                    source: kuser.faceIconUrl
                    implicitWidth: Kirigami.Units.gridUnit * 4
                    implicitHeight: implicitWidth

                    Layout.row: 1
                    Layout.column: 1
                    Layout.alignment: Qt.AlignCenter
                }

                Repeater {
                    id: channelRepeater

                    model: testOverlay.sinkObject && testOverlay.sinkObject.rawChannels

                    delegate: ToolButton {
                        readonly property bool isPlaying: tester.playingChannels.includes(modelData)
                        readonly property var channelData: {
                            switch (modelData) {
                            case "front-left": return {text: i18nd("kcm_pulseaudio", "Front Left"), row: 0, column: 0, angle: 45};
                            case "front-center": return {text: i18nd("kcm_pulseaudio", "Front Center"), row: 0, column: 1, angle: 90};
                            case "front-right": return {text: i18nd("kcm_pulseaudio", "Front Right"), row: 0, column: 2, angle: 135};
                            case "side-left": return {text: i18nd("kcm_pulseaudio", "Side Left"), row: 1, column: 0, angle: 0};
                            case "side-right": return {text: i18nd("kcm_pulseaudio", "Side Right"), row: 1, column: 2, angle: 180};
                            case "rear-left": return {text: i18nd("kcm_pulseaudio", "Rear Left"), row: 2, column: 0, angle: -45};
                            case "lfe": return {text: i18nd("kcm_pulseaudio", "Subwoofer"), row: 2, column: 1, angle: -90};
                            case "rear-right": return {text: i18nd("kcm_pulseaudio", "Rear Right"), row: 2, column: 2, angle: -135};
                            case "mono" : return {text: i18nd("kcm_pulseaudio", "Mono"), row: 0, column: 1, angle: 90};
                            }
                            // We have a non-standard channel name
                            return {
                                text: modelData,
                                row: 3 + Math.floor(index / 3),
                                // To keep the avatar centered in case of just 1 or 2 non-standard channels
                                column: (channelRepeater.count < 3 && index === channelRepeater.count - 1) ? index + 1 : index % 3,
                                angle: 0
                            }
                        }

                        Layout.row: channelData.row
                        Layout.column: channelData.column
                        Layout.fillWidth: true
                        Layout.fillHeight: true
                        Layout.preferredWidth: Kirigami.Units.gridUnit * 8
                        Layout.minimumHeight: Kirigami.Units.gridUnit * 4

                        contentItem: ColumnLayout {
                            spacing: 0

                            Kirigami.Icon {
                                source: "qrc:/kcm_pulseaudio/icons/audio-speakers-symbolic.svg"
                                isMask: true
                                color: isPlaying ? Kirigami.Theme.highlightColor : Kirigami.Theme.textColor
                                implicitWidth: Kirigami.Units.iconSizes.medium
                                implicitHeight: Kirigami.Units.iconSizes.medium
                                Layout.fillWidth: true
                                Layout.margins: Kirigami.Units.smallSpacing
                                rotation: channelData.angle
                            }

                            Label {
                                text: channelData.text
                                color: isPlaying ? Kirigami.Theme.highlightColor : Kirigami.Theme.textColor
                                Layout.fillWidth: true
                                Layout.fillHeight: true
                                Layout.margins: Kirigami.Units.smallSpacing
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignTop
                                wrapMode: Text.WordWrap
                            }
                        }

                        onClicked: {
                            testError.visible = false;
                            tester.testChannel(modelData);
                        }
                    }
                }
            }

            Label {
                text: i18nd("kcm_pulseaudio", "Click on any speaker to test sound")
                font: Kirigami.Theme.smallFont
                Layout.alignment: Qt.AlignCenter
                Layout.topMargin: Kirigami.Units.gridUnit
            }
        }
    }
}
